package cn.kdan.cloud.pdf.office.account.service.impl;

import cn.kdan.cloud.pdf.office.account.constant.UserConstant;
import cn.kdan.cloud.pdf.office.account.mapper.ext.ExtUserMapper;
import cn.kdan.cloud.pdf.office.account.model.User;
import cn.kdan.cloud.pdf.office.account.model.UserExample;
import cn.kdan.cloud.pdf.office.account.model.UserSubscriptionInfo;
import cn.kdan.cloud.pdf.office.account.model.VppCompany;
import cn.kdan.cloud.pdf.office.account.service.CompanyService;
import cn.kdan.cloud.pdf.office.account.service.UserService;
import cn.kdan.cloud.pdf.office.account.service.UserSubscriptionInfoService;
import cn.kdan.cloud.pdf.office.account.utils.RSAUtils;
import cn.kdan.cloud.pdf.office.api.account.dto.LogOffUserDTO;
import cn.kdan.cloud.pdf.office.api.account.dto.UpdateUserForOrderDTO;
import cn.kdan.cloud.pdf.office.api.account.vo.UserDetailVO;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.api.account.vo.UserPageVO;
import cn.kdan.cloud.pdf.office.common.vo.UserSubscriptionInfoVO;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.constant.RabbitMqConstant;
import cn.kdan.cloud.pdf.office.common.dto.UserRegisterDTO;
import cn.kdan.cloud.pdf.office.common.enums.EmailCodeTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.*;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.CommonUtils;
import cn.kdan.cloud.pdf.office.common.utils.RedisUtils;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
@Transactional
@Slf4j
public class UserServiceImpl implements UserService {

    @Resource
    private ExtUserMapper extUserMapper;
    @Autowired
    private CompanyService companyService;
    @Autowired
    private UserSubscriptionInfoService userSubscriptionInfoService;

    @Autowired
    private RedisUtils<String, Object> redisUtils;

    @Autowired
    private RabbitTemplate rabbitTemplate;

    @Value("${freeDate}")
    private String freeDate;


    @Override
    public UserVO getById(String userId) {
        return convert(extUserMapper.selectByPrimaryKey(userId));
    }

    @Override
    public User getUser(String userId) {
        return extUserMapper.selectByPrimaryKey(userId);
    }

    @Override
    public PageInfo<UserPageVO> page(Integer page, Integer pageSize,
                                     String selectType,
                                     List<String> appIds,
                                     List<String> roles,
                                     List<String> accountTypes,
                                     List<String> accountSources,
                                     List<String> payTypes,
                                     String queryTimeType,
                                     Date startTime,
                                     Date endTime,
                                     List<String> queryString) {
        PageHelper.startPage(page, pageSize);
        return new PageInfo<>(extUserMapper.listWithInfo(selectType,appIds,roles,accountTypes,accountSources,payTypes,queryTimeType,startTime,endTime,queryString));
    }



    @Override
    public User getByAccount(String account, String platformType) {
        List<User> list = extUserMapper.getByAccount(account ,Integer.parseInt(platformType));
        return CollectionUtils.isEmpty(list) ? null : list.get(0);
    }

    @Override
    public UserVO getByAppAccount(String account, String platformType) {
        List<User> list = extUserMapper.getByAccount(account,Integer.parseInt(platformType));
        return CollectionUtils.isEmpty(list) ? null : convert(list.get(0));
    }


    public UserVO convert(User user) {
        if(user == null){
            return new UserVO();
        }
        UserVO userVO = new UserVO();
        BeanUtils.copyProperties(user, userVO);
        return userVO;
    }

    @Override
    public UserPageVO getUserForTransfer(String userId) {
       return extUserMapper.getUserByIdWithApp(userId);
    }

    @Override
    public UserDetailVO detail(String userId) {
        UserDetailVO detailVO = new UserDetailVO();
        User user = extUserMapper.selectByPrimaryKey(userId);
        detailVO.setId(user.getId());
        detailVO.setEmail(user.getEmail());
        if(StringUtils.isNotEmpty(user.getCompanyId())){
          VppCompany company = companyService.getById(user.getCompanyId());
          detailVO.setCompanyName(company.getName());
        }
        return detailVO;
    }

    @Override
    public void updateUserForOrder(UpdateUserForOrderDTO updateUserForOrderDTO) {
        //支付成功之后修改用户状态
        User user = extUserMapper.selectByPrimaryKey(updateUserForOrderDTO.getUserId());
        //通过用户可以查到对应的app
        user.setUpdatedAt(new Date());
        //账号类型 付费账号1 免费账号0 AccountTypeEnum
        user.setAccountType(AccountTypeEnum.PAID_ACCOUNT.value());
        //通过用户id和平台类型查询会员信息表，如果有会员信息则更新，如果没有会员信息就插入
        UserSubscriptionInfo userSubscriptionInfo = userSubscriptionInfoService.getByUserIdAndPlatform(user.getId(), updateUserForOrderDTO.getPlatform());
        if(ObjectUtils.isEmpty(userSubscriptionInfo)){
            UserSubscriptionInfo newInfo = new UserSubscriptionInfo().withId(CommonUtils.generateId())
                    .withUserId(user.getId()).withStatus(updateUserForOrderDTO.getUserStatus())
                    .withPayType(updateUserForOrderDTO.getPayTpe()).withEndDate(updateUserForOrderDTO.getEndDate())
                    .withProductId(updateUserForOrderDTO.getProductId()).withAppId(updateUserForOrderDTO.getAppId())
                    .withPlatform(updateUserForOrderDTO.getPlatform());
            userSubscriptionInfoService.insert(newInfo);
        }else{
          //更新
            userSubscriptionInfo.setPayType(updateUserForOrderDTO.getPayTpe());
            userSubscriptionInfo.setStatus(updateUserForOrderDTO.getUserStatus());
            if(!ObjectUtils.isEmpty(updateUserForOrderDTO.getEndDate())){
                userSubscriptionInfo.setEndDate(updateUserForOrderDTO.getEndDate());
            }
            userSubscriptionInfoService.update(userSubscriptionInfo);
        }
        extUserMapper.updateByPrimaryKey(user);
    }


    @Override
    public void register(UserRegisterDTO userRegisterDTO) {
        //参数校验
        validRegisterParam(userRegisterDTO);
        Integer accountSource = ObjectUtils.isEmpty(userRegisterDTO.getAccountSource()) ? AccountSourceEnum.REGISTER.value() : userRegisterDTO.getAccountSource();
        initUser(userRegisterDTO.getAppId(),userRegisterDTO.getUsername(),userRegisterDTO.getPassword(), accountSource,userRegisterDTO.getModel());
    }

    private User initUser(String appId, String email, String pwd, Integer accountSource, String model) {
        User user = new User();
        user.setId(CommonUtils.generateId());
        user.setAppId(appId);
        user.setEmail(email);
        user.setCreatedAt(new Date());
        user.setAccountType(AccountTypeEnum.FREE_ACCOUNT.value());
        user.setPlatformType(PlatformTypeEnum.WEBSITE.value());
        user.setAccountSource(accountSource);
        //用户来源平台
        user.setModelSource(model);
        user.setValidFlag(PDFOfficeUserStatusEnum.VALID.value().toString());
        if(StringUtils.isNotEmpty(pwd)){
            user.setDigestPassword(new RSAUtils().encrypt(pwd));
        }
        extUserMapper.insert(user);
        return user;
    }

    @Override
    public void logOff(LogOffUserDTO logOffUserDTO) {
        User user = getUserWithCheck(logOffUserDTO.getUserId());
        handleLogOff(user,logOffUserDTO.getRemark());
    }

    @Override
    public void updateUser(User user) {
      extUserMapper.updateByPrimaryKeySelective(user);
    }

    @Override
    public void logOffForUser(String code, String userId, String appId) {
        User user = getUser(userId);
        //检查验证码是否正确
        checkEmailCodeValid(EmailCodeTypeEnum.USER_LOG_OFF, user.getEmail(), code,user.getAppId());
        //检查用户订阅（试用期可以注销，订阅期不能注销）
        List<UserSubscriptionInfo> list = userSubscriptionInfoService.listByUserId(userId);
        if(!CollectionUtils.isEmpty(list)){
            list.forEach(item->{
                //如果订阅中，则提示不能注销
                if(item.getStatus().equals(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS.value()) ){
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_USER_SUBSCRIPTION_IN_PROGRESS);
                }
            });
        }
        //注销用户 todo 注销中 三天时间删除
        log.info("rabbitMq 发送延时队列，用户三天后注销：{}", userId);
        rabbitTemplate.convertAndSend(RabbitMqConstant.USER_LOGOFF_DELAY_EXCHANGE,
                RabbitMqConstant.USER_LOGOFF_ROUTING_KEY,
                userId,
                (msg -> {
                    msg.getMessageProperties().setHeader("x-delay", UserConstant.USER_LOGOFF_DELAY_TIME + "");
                    return msg;
                }));
        //注销中
        user.setUpdatedAt(new Date());
        user.setValidFlag(PDFOfficeUserStatusEnum.LOGGED_OUT_ING.value().toString());
        extUserMapper.updateByPrimaryKeySelective(user);
    }

    @Override
    public UserInfoVO getInfoById(String id) {
        UserInfoVO infoVO = new UserInfoVO();
        User user = getUser(id);
        BeanUtils.copyProperties(user,infoVO);
        List<UserSubscriptionInfo> list = userSubscriptionInfoService.listByUserId(id);
        List<UserSubscriptionInfoVO> voList = new ArrayList<>();
        for (UserSubscriptionInfo userSubscriptionInfo : list) {
            UserSubscriptionInfoVO vo = new UserSubscriptionInfoVO();
            BeanUtils.copyProperties(userSubscriptionInfo,vo);
            voList.add(vo);
        }
        if(!ObjectUtils.isEmpty(freeDate)){
            try {
                Date date = CommonUtils.parseDate("yyyy-MM-dd HH:mm:ss",freeDate);
                infoVO.setFreeDate(date);
                infoVO.setIsInFreeUseTime(Boolean.TRUE);
            } catch (ParseException e) {
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_TIME_TRANSFER_ERROR);
            }
        }
        infoVO.setSubscriptionInfoList(voList);
        return infoVO;
    }

    /**
     * 注销公共方法
     *
     * @param user 用户
     */
    @Override
    public void handleLogOff(User user, String remark) {
        user.setRemark(remark);
        user.setUpdatedAt(new Date());
        user.setValidFlag(PDFOfficeUserStatusEnum.LOGGED_OUT.value().toString());
        extUserMapper.updateByPrimaryKeySelective(user);
    }

    @Override
    public UserVO getByEmail(String email) {
        UserExample userExample = new UserExample();
        userExample.createCriteria().andEmailEqualTo(email)
                .andValidFlagNotEqualTo(PDFOfficeUserStatusEnum.LOGGED_OUT.value().toString());
        List<User> users = extUserMapper.selectByExample(userExample);
        if (CollectionUtils.isEmpty(users) || users.size() <= 0){
            return null;
        } else {
            User user = users.get(0);
            UserVO userVO = new UserVO();
            BeanUtils.copyProperties(user,userVO);
            return userVO;
        }
    }

    private User getUserWithCheck(String userId) {
        User user = extUserMapper.selectByPrimaryKey(userId);
        if(ObjectUtils.isEmpty(user)){
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_EMAIL_NOT_REGISTER);
        }
        return user;
    }

    private void validRegisterParam(UserRegisterDTO param) {
        //邮箱是否被注册
        if (!ObjectUtils.isEmpty(getByAppAccount(param.getAppId(), param.getPlatformType()))) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_EMAIL_EXIST);
        }
        //验证邮箱格式
        String emailRegex = CommonConstant.emailRegex;
        if (!param.getUsername().matches(emailRegex)) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_EMAIL_INVALID);
        }
        if(StringUtils.isNotEmpty(param.getVerifyCode())){
            //邮件验证码错误
            checkEmailCodeValid(EmailCodeTypeEnum.USER_REGISTER, param.getUsername(), param.getVerifyCode(), param.getAppId());
        }
    }



    private void checkEmailCodeValid(EmailCodeTypeEnum type, String account, String code, String appId) {
        //获取用户存在redis中的登录邮箱验证码
        String captchaCode = redisUtils.hget(CommonConstant.EMAIL_VERIFY_CODE_KEY + type.value()  + CommonConstant.STRIKE_THROUGH + appId , account);
        if(StringUtils.isNotEmpty(code) && code.equals(captchaCode)){
            //验证通过删除
            redisUtils.hdel(CommonConstant.EMAIL_VERIFY_CODE_KEY + type.value() + CommonConstant.STRIKE_THROUGH + appId  , account);
        }else{
            throw new BackendRuntimeException(ExceptionEnum.EMAIL_VERIFY_CODE_KEY_ERROR);
        }
    }




}
