package cn.kdan.cloud.pdf.office.api.payment.feign;

import cn.kdan.cloud.pdf.office.api.payment.dto.*;
import cn.kdan.cloud.pdf.office.api.payment.feign.hystrix.OrderHystrix;
import cn.kdan.cloud.pdf.office.api.payment.vo.CreateOrderResultVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrderUpdateVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import javax.validation.constraints.NotNull;
import java.util.List;


@Component
@FeignClient(value = CommonConstant.PAYMENT_PROVIDER_NAME + "/order", fallback = OrderHystrix.class)
public interface OrderApi {

    /**
     * 创建支付订单
     *
     * @param createOrderDTO createOrderDTO
     * @return CreateOrderResultVO
     */
    @PostMapping("/createOrder")
    ResultMap<CreateOrderResultVO> createOrder(@Validated @RequestBody CreateOrderDTO createOrderDTO);

    /**
     * 订单创建（applePay专用）
     *
     * @param createOrderDTO createOrderDTO
     * @return CreateOrderResultVO
     */
    @PostMapping("/appStoreCreateOrder")
    ResultMap<CreateOrderResultVO> appStoreCreateOrder(@Validated @RequestBody CreateOrderDTO createOrderDTO);

    /**
     * 订单支付成功（applePay专用）
     * 回传appal pay id &签名证书
     *
     * @param appStoreOrderSucceed appStoreOrderSucceed
     * @return Void
     */
    @PostMapping("/appStoreOrderSucceed")
    ResultMap<UserInfoVO> appStoreOrderSucceed(@Validated @RequestBody AppStoreOrderSucceedDTO appStoreOrderSucceed);

    @PostMapping("/appStoreBuyComplete")
    ResultMap<Void> appStoreBuyComplete(@Validated @RequestBody AppStoreBuyCompleteDTO dto);

    /**
     * appStore 权益校验
     *
     * @param equityVerificationDTO equityVerificationDTO
     * @return UserInfoVO
     */
    @PostMapping("/appStoreEquityVerification")
    ResultMap<UserInfoVO> appStoreEquityVerification(@RequestBody EquityVerificationDTO equityVerificationDTO);

    /**
     * 手动创建虚拟订单
     *
     * @param createOrderManual createOrderManual
     * @return OrderUpdateVO
     */
    @PostMapping("/createOrderManual")
    ResultMap<OrderUpdateVO> createOrderManual(@Validated @RequestBody CreateOrderManualDTO createOrderManual);

    /**
     * 通过id获取订单信息
     *
     * @param orderId orderId
     * @return Orders
     */
    @GetMapping("/getOrderById")
    ResultMap<OrdersVO> getOrderById(@Validated @NotNull(message = "orderId不能为空")
                                     @RequestParam("orderId") String orderId);

    /**
     * 通过id获取订单信息
     *
     * @param orderId orderId
     * @return Orders
     */
    @GetMapping("/getOrderWithUpdate")
    ResultMap<OrdersVO> getOrderWithUpdate(@Validated @NotNull(message = "orderId不能为空")
                                     @RequestParam("orderId") String orderId);

    /**
     * 通过自定义订单id获取订单信息
     *
     * @param tradeNo tradeNo
     * @return Orders
     */
    @GetMapping("/getOrderByTradeNo")
    ResultMap<List<OrdersVO>> getOrderByTradeNo(@Validated @NotNull(message = "tradeNo不能为空")
                                          @RequestParam("tradeNo") String tradeNo);

    /**
     * 手动推单
     *
     * @param chargebackOrderDTO chargebackOrderDTO
     * @return Void
     */
    @PostMapping("/chargebackOrder")
    ResultMap<Void> chargebackOrder(@RequestBody ChargebackOrderDTO chargebackOrderDTO);


    /**
     * 取消续订
     *
     * @param cancelSubscription cancelSubscription
     * @return Void
     */
    @PostMapping("/cancelSubscription")
    ResultMap<Void> cancelSubscription(@Validated @RequestBody CancelSubscriptionDTO cancelSubscription);

}
