package cn.kdan.cloud.pdf.office.payment.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.feign.UserSubscriptionInfoApi;
import cn.kdan.cloud.pdf.office.api.account.vo.UserSubscriptionInfo;
import cn.kdan.cloud.pdf.office.api.email.bo.EmailSendBO;
import cn.kdan.cloud.pdf.office.api.email.feign.EmailApi;
import cn.kdan.cloud.pdf.office.api.payment.appstore.InApp;
import cn.kdan.cloud.pdf.office.api.payment.bo.OrderSucceededBO;
import cn.kdan.cloud.pdf.office.api.payment.constant.OrderConstant;
import cn.kdan.cloud.pdf.office.api.payment.constant.SubscriptionConstant;
import cn.kdan.cloud.pdf.office.api.payment.dto.AppStoreBuyCompleteDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreateSubscriptionDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.EquityVerificationDTO;
import cn.kdan.cloud.pdf.office.api.payment.enums.PaymentMethodEnum;
import cn.kdan.cloud.pdf.office.api.payment.vo.CreateOrderResultVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionsVO;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.enums.EmailCodeTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserSubscriptionPayTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserSubscriptionStatusEnum;
import cn.kdan.cloud.pdf.office.common.enums.payment.SubscriptionTypeEnum;
import cn.kdan.cloud.pdf.office.common.utils.CommonUtils;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.api.payment.appstore.AppStoreReceiptInfo;
import cn.kdan.cloud.pdf.office.api.payment.constant.AppStoreAPIConstant;
import cn.kdan.cloud.pdf.office.api.payment.dto.AppStoreOrderSucceedDTO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import cn.kdan.cloud.pdf.office.payment.client.AppStoreClient;
import cn.kdan.cloud.pdf.office.payment.entity.Order;
import cn.kdan.cloud.pdf.office.payment.entity.RestorePurchaseLogs;
import cn.kdan.cloud.pdf.office.payment.entity.Subscription;
import cn.kdan.cloud.pdf.office.payment.properties.AppStoreProperties;
import cn.kdan.cloud.pdf.office.payment.service.AppStoreService;
import cn.kdan.cloud.pdf.office.payment.service.OrderService;
import cn.kdan.cloud.pdf.office.payment.service.RestorePurchaseLogsService;
import cn.kdan.cloud.pdf.office.payment.service.SubscriptionsService;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author ComPDFKit-WPH 2023/6/29
 *
 * AppStoreServiceImpl
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class AppStoreServiceImpl implements AppStoreService {

    private final AppStoreClient appStoreClient;

    private final OrderService orderService;

    private final SubscriptionsService subscriptionsService;

    private final WebhookServiceImpl webhookService;

    private final ProductApi productApi;

    private final UserApi userApi;

    private final AppStoreProperties appStoreProperties;

    private final UserSubscriptionInfoApi userSubscriptionInfoApi;

    private final RestorePurchaseLogsService restorePurchaseLogsService;

    private final EmailApi emailApi;

    @Transactional(noRollbackFor = BackendRuntimeException.class)
    @Override
    public UserInfoVO appStoreOrderSucceed(AppStoreOrderSucceedDTO appStoreOrderSucceed) {
        log.info("AppStore 支付成功信息订单id：{}，产品信息：{},票据信息：{},\n restore：{}，",appStoreOrderSucceed.getOrderId(),appStoreOrderSucceed.getApplePayProductId(),appStoreOrderSucceed.getReceipt(),appStoreOrderSucceed.getRestore());
        // 恢复购买
        if (1 == appStoreOrderSucceed.getRestore()){
            return this.appStoreRestorePurchase(appStoreOrderSucceed);
        }

        try {
            OrdersVO orderVO = orderService.getOrderById(appStoreOrderSucceed.getOrderId());
            if (ObjectUtils.isEmpty(orderVO)) {
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_ORDER_DOES_NOT_EXIST);
            }
            if (!orderVO.getStatus().equals(OrderConstant.CREATED)){
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_THE_ORDER_STATUS_IS_ABNORMAL);
            }
            // 插入第三方订单id和票据信息
            Order orderChangeResult = new Order();
            orderChangeResult.setId(appStoreOrderSucceed.getOrderId());
            orderChangeResult.setResult(appStoreOrderSucceed.getReceipt());
            orderService.updateById(orderChangeResult);
            // 校验是否是同一产品id  r_product_pay表
            appStoreOrderSucceed.setRetry(true);
            List<InApp> inAppFilters = appStoreReceiptVerify(appStoreOrderSucceed);
            if (CollectionUtils.isEmpty(inAppFilters)) {
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_APP_STORE_TICKET_VALIDATION_FAILED);
            }

            log.info("当前订单数据：{}",orderVO);
            InApp inApp = inAppFilters.get(0);

            if (StringUtils.isNotEmpty(inApp.getCancellationReason())) {
                // order交易失败
                log.info("order 取消交易，订单id，{}，取消原因：{},\n 值1表示客户由于应用程序内的实际或感知问题而取消了交易。值0表示交易因其他原因被取消；例如，如果客户意外购买",appStoreOrderSucceed.getOrderId(),inApp.getCancellationReason());
//                orderService.closeOrder(appStoreOrderSucceed.getOrderId());
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_USER_TRANSACTION_CANCELED_OR_FAILED);
            }

            String transactionId = inApp.getTransactionId();
            if (ObjectUtils.isNotEmpty(orderService.getOrderByThirdOrderNo(transactionId))) {
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_TRANSACTION_ID_REPEAT);
            }
            String originalTransactionId = inApp.getOriginalTransactionId();

            Order orderChangeThirdTradeNo = new Order();
            orderChangeThirdTradeNo.setThirdTradeNo(originalTransactionId);
            orderChangeThirdTradeNo.setId(appStoreOrderSucceed.getOrderId());
            orderService.updateById(orderChangeThirdTradeNo);

            BigDecimal price = inApp.getTransactionId().equals(inApp.getOriginalTransactionId()) ? appStoreProperties.getIsDiscountPrice(): appStoreProperties.getNotDiscountPrice();
            // 添加订阅信息
            webhookService.handlingUserSubscriptionActivation(price,originalTransactionId,transactionId,orderVO,null);
            // 返回订阅信息
            UserInfoVO result = userApi.getInfoById(orderVO.getUserId()).getResult();
            result.setDigestPassword(null);
            return result;
        } catch (BackendRuntimeException e) {
            orderService.closeOrder(appStoreOrderSucceed.getOrderId());
            throw e;
        }
    }

    @Transactional
    @Override
    public void appStoreBuyComplete(AppStoreBuyCompleteDTO dto) {
        log.info("调用购买完成接口参数：{}", dto.toString());
        // 校验票据
        AppStoreOrderSucceedDTO appStoreOrderSucceedDTO = new AppStoreOrderSucceedDTO();
        appStoreOrderSucceedDTO.setReceipt(dto.getReceipt());
        appStoreOrderSucceedDTO.setApplePayProductId(dto.getApplePayProductId());
        List<InApp> inAppFilters = appStoreReceiptVerify(appStoreOrderSucceedDTO);
        if (CollectionUtils.isEmpty(inAppFilters)) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_APP_STORE_TICKET_VALIDATION_FAILED);
        }
        InApp inApp = inAppFilters.get(0);
        // 查询票据订单是否存在
        OrdersVO orderByThirdOrderNo = orderService.getOrderByThirdOrderNo(inApp.getTransactionId());
        if (BeanUtil.isNotEmpty(orderByThirdOrderNo)) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_TICKET_HAS_BEEN_CREATED);
        }

        // 创建订单
        log.info("创建订单;");
        String orderId = CommonUtils.generateId();
        Order order = new Order();
        order.setId(orderId);
        order.setEmail(userApi.getById(dto.getUserId()).getResult().getEmail());
        order.setAppId(dto.getAppId());
        order.setUserId(StringUtils.isNotEmpty(dto.getUserId()) ? dto.getUserId() : "0");
        order.setProductId(dto.getProductId());
        order.setPayment(dto.getPaymentMethod().getValue());
        order.setStatus(OrderConstant.CREATED);
        order.setInvoiceNo(MyDateUtils.getTimeStamp() + (int) ((Math.random() * 9 + 1) * 1000));
//        orders.setDiscount(createOrderDTO.getDiscount());
        String readeNo = MyDateUtils.getTimeStamp() + "-" + dto.getProductId() + "-" + (int) ((Math.random() * 9 + 1) * 1000);
        order.setTradeNo(readeNo);
        //订单类型3appstore购买
        order.setType(3);
        order.setResult(dto.getReceipt());
        order.setThirdTradeNo(inApp.getOriginalTransactionId());
        order.setThirdOrderNo(inApp.getTransactionId());
        order.setPrice(inApp.getTransactionId().equals(inApp.getOriginalTransactionId()) ? appStoreProperties.getIsDiscountPrice(): appStoreProperties.getNotDiscountPrice());
        order.setPayDate(new Date());
        order.setStatus(OrderConstant.COMPLETED);

        // 订阅--由用户创建订阅信息，并关联，如果用户有权益先解绑
        if (StringUtils.isNotEmpty(dto.getUserId())) {
            log.info("购买完成、订单用户关联，订单：{}；用户：{}",orderId, dto.getUserId());
            String subId = CommonUtils.generateId();
            order.setSubscriptionId(subId);
            // 创建订阅
            CreateSubscriptionDTO createSubscription = new CreateSubscriptionDTO();
            createSubscription.setId(subId);
            createSubscription.setUserId(order.getUserId());
            createSubscription.setOrderId(orderId);
            createSubscription.setAppId(order.getAppId());
            createSubscription.setPayment(order.getPayment());
            createSubscription.setProductId(order.getProductId());
            createSubscription.setThirdSubscriptionId(order.getThirdTradeNo());
            createSubscription.setEmail(order.getEmail());
            createSubscription.setBuyerType(4);
            createSubscription.setSubscriptionType(SubscriptionTypeEnum.MEMBER.value());
            subscriptionsService.createSubscription(createSubscription);
        }
        orderService.insertOrder(order);
    }

    @Transactional
    @Override
    public UserInfoVO appStoreEquityVerification(EquityVerificationDTO equityVerificationDTO) {
        // 没有票据直接返回用户
        if (StringUtils.isEmpty(equityVerificationDTO.getReceipt())) {
            UserInfoVO result = userApi.getInfoById(equityVerificationDTO.getUserId()).getResult();
            result.setDigestPassword(null);
            return result;
        }
        log.info("appStore 权益校验,用户id：{}",equityVerificationDTO.getUserId());
        AppStoreOrderSucceedDTO appStoreOrderSucceedDTO = new AppStoreOrderSucceedDTO();
        appStoreOrderSucceedDTO.setReceipt(equityVerificationDTO.getReceipt());
        appStoreOrderSucceedDTO.setApplePayProductId(equityVerificationDTO.getApplePayProductId());
        List<InApp> inApps = this.appStoreReceiptVerify(appStoreOrderSucceedDTO);
        if (CollectionUtils.isEmpty(inApps)) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_APP_STORE_TICKET_VALIDATION_FAILED);
        }
        InApp inApp = inApps.get(0);
        // 获取票据订单信息
        String transactionId = inApp.getTransactionId();
        String originalTransactionId = inApp.getOriginalTransactionId();
        BigDecimal price = inApp.getTransactionId().equals(inApp.getOriginalTransactionId()) ? appStoreProperties.getIsDiscountPrice(): appStoreProperties.getNotDiscountPrice();
        OrdersVO orderByThirdOrderNo = orderService.getOrderByThirdOrderNo(transactionId);
        if (null == orderByThirdOrderNo) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_CURRENT_ORDER_NOT_RECORDED);
        }
        // 判断票据有没有绑定
        if (StringUtils.isEmpty(orderByThirdOrderNo.getUserId()) || "0".equals(orderByThirdOrderNo.getUserId())) {
            // 没有绑定直接建立新的绑定关系
            log.info("新购买订单，票据{}，用户绑定{}", equityVerificationDTO.getReceipt(), equityVerificationDTO.getUserId());
            // 新增新用户的订阅
            CreateSubscriptionDTO createSubscription = new CreateSubscriptionDTO();
            String subscriptionId = CommonUtils.generateId();
            createSubscription.setId(subscriptionId);
            createSubscription.setUserId(equityVerificationDTO.getUserId());
            // 使用购买的订单号
            createSubscription.setOrderId(orderByThirdOrderNo.getId());
            createSubscription.setAppId(orderByThirdOrderNo.getAppId());
            createSubscription.setPayment(orderByThirdOrderNo.getPayment());
            createSubscription.setProductId(orderByThirdOrderNo.getProductId());
            createSubscription.setThirdSubscriptionId(originalTransactionId);
            createSubscription.setEmail(userApi.getById(equityVerificationDTO.getUserId()).getResult().getEmail());
            createSubscription.setBuyerType(4);//2官网购买 3后台赠送 4appstore
            createSubscription.setSubscriptionType(SubscriptionTypeEnum.MEMBER.value());//如果是0则是试用
            subscriptionsService.createSubscription(createSubscription);
            // 更新订单
            OrderSucceededBO orderSucceeded = OrderSucceededBO.builder()
                    .thirdTradeNo(originalTransactionId)
                    .thirdOrderNo(transactionId)
                    .price(price)
                    .subscriptionId(subscriptionId)
                    .userId(equityVerificationDTO.getUserId())
                    .orderId(orderByThirdOrderNo.getId())
                    .build();
            if (PaymentMethodEnum.APPLE_PAY.getValue().equals(orderByThirdOrderNo.getPayment())) {
                orderSucceeded.setOrderId(orderByThirdOrderNo.getId());
            }
            orderService.updateOrderSucceeded(orderSucceeded);
        } else {
            // 有绑定的票据，判断是否当前用户绑定
            log.info("票据已绑定{}，原用户{}；当前用户{}",equityVerificationDTO.getReceipt(),
                    orderByThirdOrderNo.getUserId(), equityVerificationDTO.getUserId());
            String oldUserId = orderByThirdOrderNo.getUserId();
            String newUserId = equityVerificationDTO.getUserId();
            if (newUserId.equals(oldUserId)) {
                // 当前用户绑定，不用处理，直接返回用户信息
                log.info("当前用户绑定，不用处理");
                UserInfoVO result = userApi.getInfoById(equityVerificationDTO.getUserId()).getResult();
                result.setDigestPassword(null);
                return result;
            }
            SubscriptionsVO subscription = subscriptionsService.getSubscriptionBySubscriptionId(orderByThirdOrderNo.getSubscriptionId());
            if (null != subscription.getIsLock() && 1 == subscription.getIsLock()) {
                log.info("当前票据已封锁");
                // 已封锁的抛出
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_TICKET_HAS_BEEN_BLOCKED);
            }
            // 判断当前票据restore次数(不统计解封前的)
            List<RestorePurchaseLogs> byOrderReceipt = restorePurchaseLogsService.getByOrderReceipt(orderByThirdOrderNo.getId());
            Integer isLock = 0;
            Integer restoreTypeNew = 0;
            if (CollectionUtils.isNotEmpty(byOrderReceipt)) {
                RestorePurchaseLogs restorePurchaseLogs = byOrderReceipt.stream().max(Comparator.comparing(RestorePurchaseLogs::getCreatedAt)).orElse(null);
                Integer restoreType = null != restorePurchaseLogs ? restorePurchaseLogs.getRestoreType() : 0;
                // 判断票据有无警告或者封锁
                if (0 != restoreType) {
                    if (1 == restoreType) {
                        log.info("当前票据已发送警告邮件");
                        // 警告的再次restore即封锁
                        restoreTypeNew = 2;
                        isLock = 1;
                    }
                    if (2 == restoreType) {
                        log.info("当前票据已封锁");
                        // 已封锁的抛出
                        throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_TICKET_HAS_BEEN_BLOCKED);
                    }
                } else {
                    // 没警告的判断近七天restore次数
                    DateTime dateTime = DateUtil.offsetDay(DateUtil.date(), -7);
                    long count = byOrderReceipt.stream().filter(c -> dateTime.isBefore(DateUtil.date(c.getCreatedAt()))).count();
                    log.info("当前票据近七天restore次数：{}", count);
                    // 达到警告邮件次数，已经有4条记录到第五次就警告
                    if (count >= 1) {
                        restoreTypeNew = 1;
                    }
                }
            }

            // 其他用户绑定，移除以前的订阅
            log.info("restore:票据第三方交易id：{}", originalTransactionId);
            subscriptionsService.updateSubscriptionStatus(orderByThirdOrderNo.getSubscriptionId(), SubscriptionConstant.expired);
            // 根据用户id和产品id修改用户订阅状态 不是自动订阅
            UserSubscriptionInfo userSubscriptionInfo = userSubscriptionInfoApi.getByUserIdAndProductId(oldUserId, orderByThirdOrderNo.getProductId()).getResult();
            UserSubscriptionInfo info = new UserSubscriptionInfo();
            info.setUserId(oldUserId);
            info.setProductId(orderByThirdOrderNo.getProductId());
            info.setPayType(PDFOfficeUserSubscriptionPayTypeEnum.NON_AUTO_SUBSCRIPTION.value());
            info.setStatus(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_EXPIRED.value());
            info.setId(userSubscriptionInfo.getId());
            userSubscriptionInfoApi.update(info);

            // 新增新用户的订阅
            CreateSubscriptionDTO createSubscription = new CreateSubscriptionDTO();
            createSubscription.setUserId(newUserId);
            String subscriptionId = CommonUtils.generateId();
            createSubscription.setId(subscriptionId);
            // 使用恢复购买的额订单号
            createSubscription.setOrderId(orderByThirdOrderNo.getId());
            createSubscription.setAppId(orderByThirdOrderNo.getAppId());
            createSubscription.setPayment(orderByThirdOrderNo.getPayment());
            createSubscription.setProductId(orderByThirdOrderNo.getProductId());
            createSubscription.setThirdSubscriptionId(originalTransactionId);
            createSubscription.setEmail(userApi.getById(newUserId).getResult().getEmail());
            createSubscription.setBuyerType(4);//2官网购买 3后台赠送 4appstore
            createSubscription.setSubscriptionType(SubscriptionTypeEnum.MEMBER.value());//如果是0则是试用
            createSubscription.setEndDate(MyDateUtils.dateToLocalDateTime(subscription.getEndDate()));
            createSubscription.setStartDate(MyDateUtils.dateToLocalDateTime(subscription.getStartDate()));
            createSubscription.setIsLock(isLock);
            subscriptionsService.createSubscription(createSubscription);
            // 更新订单
            OrderSucceededBO orderSucceeded = OrderSucceededBO.builder()
                    .thirdTradeNo(originalTransactionId)
                    .thirdOrderNo(transactionId)
                    .price(price)
                    .subscriptionId(subscriptionId)
                    .userId(newUserId)
                    .orderId(orderByThirdOrderNo.getId())
                    .build();
            if (PaymentMethodEnum.APPLE_PAY.getValue().equals(orderByThirdOrderNo.getPayment())) {
                orderSucceeded.setOrderId(orderByThirdOrderNo.getId());
            }
            orderService.updateOrderSucceeded(orderSucceeded);

            // 记录恢复购买 restore_purchase_logs
            RestorePurchaseLogs restorePurchaseLogs = new RestorePurchaseLogs();
            restorePurchaseLogs.setOldUserId(oldUserId);
            restorePurchaseLogs.setNewUserId(newUserId);
            restorePurchaseLogs.setAppleProductId(equityVerificationDTO.getApplePayProductId());
            restorePurchaseLogs.setTransactionId(transactionId);
            restorePurchaseLogs.setOriginalTransactionId(originalTransactionId);
            restorePurchaseLogs.setReceipt(equityVerificationDTO.getReceipt());
            restorePurchaseLogs.setOrderId(orderByThirdOrderNo.getId());
            restorePurchaseLogs.setNewSubscriptionId(subscriptionId);
            restorePurchaseLogs.setRestoreType(restoreTypeNew);
            restorePurchaseLogsService.save(restorePurchaseLogs);

            // 发送邮件警告
            if (restoreTypeNew == 1) {
                log.info("发送警告邮件，e-mail:{}；subscriptionId:{}", createSubscription.getEmail(), subscriptionId);
                //发送邮件给用户
                EmailSendBO bo = new EmailSendBO();
                bo.setToEmail(createSubscription.getEmail());
                bo.setUseEvent(EmailCodeTypeEnum.ACCOUNT_WARMING.value());
                //设置title
                Map<String,String> titleMap = new HashMap<>();
                bo.setSendTitleContent(titleMap);
                Map<String,String> contentMap = new HashMap<>();
                bo.setSendContent(contentMap);
                emailApi.sendEmail(bo);
            }
            // 发送封锁邮件
            if (restoreTypeNew == 2) {
                log.info("发送封锁邮件，e-mail:{}；subscriptionId:{}", createSubscription.getEmail(), subscriptionId);
                //发送邮件给用户
                EmailSendBO bo = new EmailSendBO();
                bo.setToEmail(createSubscription.getEmail());
                bo.setUseEvent(EmailCodeTypeEnum.ACCOUNT_LOCKED.value());
                //设置title
                Map<String,String> titleMap = new HashMap<>();
                bo.setSendTitleContent(titleMap);
                Map<String,String> contentMap = new HashMap<>();
                bo.setSendContent(contentMap);
                emailApi.sendEmail(bo);
            }
        }

        UserInfoVO result = userApi.getInfoById(equityVerificationDTO.getUserId()).getResult();
        result.setDigestPassword(null);
        return result;
    }

    /**
     * 票据验证
     *
     * @param appStoreOrderSucceed appStoreOrderSucceed
     * @return List<InApp>
     */
    private List<InApp> appStoreReceiptVerify(AppStoreOrderSucceedDTO appStoreOrderSucceed) {
        // 验证票据信息
        AppStoreReceiptInfo appStoreReceiptInfo = appStoreClient.verifyReceipt(appStoreOrderSucceed);
        log.info("票据信息：{}",appStoreReceiptInfo.getReceipt().getInApp());

        List<InApp> inAppFilters = appStoreReceiptInfo.getReceipt().getInApp().stream().filter(app -> {
            if (app.getProductId().equals(appStoreOrderSucceed.getApplePayProductId())) {
                String expiresDateMs = app.getExpiresDateMs();
                String inAppOwnershipType = app.getInAppOwnershipType();
                // 校验是否是支付成功的订单
                if (!AppStoreAPIConstant.INAPPOWNERSHIPTYPE_PURCHASED.equals(inAppOwnershipType)) {
                    return false;
                }
                // 校验订阅到期时间是否在当前时间之后
                long expiresDateMsL = Long.parseLong(expiresDateMs);
                // 从时间戳创建Instant对象
                Instant instant = Instant.ofEpochMilli(expiresDateMsL);
                LocalDateTime endTime = LocalDateTime.ofInstant(instant, ZoneId.of("Etc/GMT+0"));
                LocalDateTime now = LocalDateTime.now(ZoneId.of("Etc/GMT+0"));
                return now.isBefore(endTime);
            }
            return false;
        }).collect(Collectors.toList());
        log.info("过滤之后票据信息：{}",inAppFilters);
        return inAppFilters;
    }

    /**
     * 恢复购买
     *
     * @param appStoreOrderSucceed appStoreOrderSucceed
     * @return UserInfoVO
     */
    private UserInfoVO appStoreRestorePurchase(AppStoreOrderSucceedDTO appStoreOrderSucceed) {
        List<InApp> inAppFilters = appStoreReceiptVerify(appStoreOrderSucceed);
        if (CollectionUtils.isEmpty(inAppFilters)) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_APP_STORE_TICKET_VALIDATION_FAILED);
        }
        InApp inApp = inAppFilters.get(0);
        if (StringUtils.isNotEmpty(inApp.getCancellationReason())) {
            log.info("order 取消交易，订单id，{}，取消原因：{},\n 值1表示客户由于应用程序内的实际或感知问题而取消了交易。值0表示交易因其他原因被取消；例如，如果客户意外购买",appStoreOrderSucceed.getOrderId(),inApp.getCancellationReason());
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_USER_TRANSACTION_CANCELED_OR_FAILED);
        }
        String transactionId = inApp.getTransactionId();
        String originalTransactionId = inApp.getOriginalTransactionId();
        OrdersVO ordersVO = orderService.getOrderByThirdOrderNo(transactionId);
        if (ObjectUtils.isEmpty(ordersVO)) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_CURRENT_ORDER_NOT_RECORDED);
        }

        // 判断是否是同一个用户
        String oldUserId = ordersVO.getUserId();
        String newUserId = appStoreOrderSucceed.getUserId();
        RestorePurchaseLogs restorePurchaseLog = restorePurchaseLogsService.getByOrderId(ordersVO.getId());
        if (ObjectUtils.isNotEmpty(restorePurchaseLog)) {
            oldUserId = restorePurchaseLog.getNewUserId();
        }
        log.info("恢复购买：oldUserId：{}，newUserId：{}",oldUserId,newUserId);
        if (org.springframework.util.ObjectUtils.nullSafeEquals(oldUserId,newUserId)){
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_THE_SAME_USER);
        }
        // 权益转移和记录
        // 移除以前的订阅
        subscriptionsService.updateSubscriptionStatus(ordersVO.getSubscriptionId(), SubscriptionConstant.expired);
        // 根据用户id和产品id修改用户订阅状态 不是自动订阅
        UserSubscriptionInfo userSubscriptionInfo = userSubscriptionInfoApi.getByUserIdAndProductId(oldUserId, ordersVO.getProductId()).getResult();
        UserSubscriptionInfo info = new UserSubscriptionInfo();
        info.setUserId(oldUserId);
        info.setProductId(ordersVO.getProductId());
        info.setPayType(PDFOfficeUserSubscriptionPayTypeEnum.NON_AUTO_SUBSCRIPTION.value());
        info.setStatus(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_EXPIRED.value());
        info.setId(userSubscriptionInfo.getId());
        userSubscriptionInfoApi.update(info);
        // 新增新用户的订阅
        SubscriptionsVO subscription = subscriptionsService.getSubscriptionBySubscriptionId(ordersVO.getSubscriptionId());

        CreateSubscriptionDTO createSubscription = new CreateSubscriptionDTO();
        createSubscription.setUserId(newUserId);
        String subscriptionId = CommonUtils.generateId();
        createSubscription.setId(subscriptionId);
        // 使用恢复购买的额订单号
        createSubscription.setOrderId(ordersVO.getId());

        createSubscription.setAppId(ordersVO.getAppId());
        createSubscription.setPayment(ordersVO.getPayment());
        createSubscription.setProductId(ordersVO.getProductId());
        createSubscription.setThirdSubscriptionId(originalTransactionId);
        createSubscription.setEmail(userApi.getById(newUserId).getResult().getEmail());
        createSubscription.setBuyerType(4);//2官网购买 3后台赠送 4appstore
        createSubscription.setSubscriptionType(SubscriptionTypeEnum.MEMBER.value());//如果是0则是试用

        createSubscription.setEndDate(MyDateUtils.dateToLocalDateTime(subscription.getEndDate()));
        createSubscription.setStartDate(MyDateUtils.dateToLocalDateTime(subscription.getStartDate()));
        subscriptionsService.createSubscription(createSubscription);

        // 记录恢复购买 restore_purchase_logs
        RestorePurchaseLogs restorePurchaseLogs = new RestorePurchaseLogs();
        restorePurchaseLogs.setOldUserId(oldUserId);
        restorePurchaseLogs.setNewUserId(newUserId);
        restorePurchaseLogs.setAppleProductId(appStoreOrderSucceed.getApplePayProductId());
        restorePurchaseLogs.setTransactionId(transactionId);
        restorePurchaseLogs.setOriginalTransactionId(originalTransactionId);
        restorePurchaseLogs.setReceipt(appStoreOrderSucceed.getReceipt());
        restorePurchaseLogs.setOrderId(ordersVO.getId());
        restorePurchaseLogs.setNewSubscriptionId(subscriptionId);
        restorePurchaseLogsService.save(restorePurchaseLogs);
        // 返回订阅信息
        UserInfoVO result = userApi.getInfoById(appStoreOrderSucceed.getUserId()).getResult();
        result.setDigestPassword(null);
        return result;
    }

}
