package cn.kdan.cloud.pdf.office.payment.service.impl;

import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.payment.constant.OrderConstant;
import cn.kdan.cloud.pdf.office.api.payment.dto.PaddleWebhookDTO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.payment.entity.RestorePurchaseLogs;
import cn.kdan.cloud.pdf.office.payment.properties.AppStoreProperties;
import cn.kdan.cloud.pdf.office.payment.service.*;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.List;

/**
 * @author ComPDFKit-WPH 2023/1/30
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class AppStoreWebhookServiceImpl implements AppStoreWebhookService {

    private final OrderService ordersService;
    private final WebhookService webhookService;
    private final RestorePurchaseLogsService restorePurchaseLogsService;
    private final AppStoreProperties appStoreProperties;
    private final UserApi userApi;

    @Override
    public void subscriptionPaymentSucceeded(Boolean isDiscount, String thirdTradeNo,String thirdOrderId) {
        log.info("appstore 订阅的支付订单已成功 appstore订单id：{}", thirdOrderId);
        OrdersVO ordersVO = ordersService.getOrderByThirdOrderNo(thirdOrderId);
        if (ObjectUtils.isNotEmpty(ordersVO) && OrderConstant.COMPLETED.equals(ordersVO.getStatus())) {
            return;
        }
        if (CollectionUtils.isEmpty(ordersService.getOrderByTradeNo(thirdTradeNo))){
            return;
        }
        BigDecimal price = isDiscount ? appStoreProperties.getIsDiscountPrice() : appStoreProperties.getNotDiscountPrice();

        List<OrdersVO> ordersVOS = ordersService.getOrderByTradeNo(thirdTradeNo);
        OrdersVO ordersVONew = ordersVOS.get(0);
        RestorePurchaseLogs restorePurchaseLogs = restorePurchaseLogsService.getByOrderId(ordersVONew.getId());
        if (ObjectUtils.isNotEmpty(restorePurchaseLogs)){
            // 该订单是产生过恢复购买
            ordersVONew.setUserId(restorePurchaseLogs.getNewUserId());
            ordersVONew.setEmail(userApi.getById(restorePurchaseLogs.getNewUserId()).getResult().getEmail());
        }
        webhookService.handleSubsequentAutomaticDeduction(price, thirdTradeNo, thirdOrderId,ordersVONew);
    }

    @Override
    public void subscriptionCancel(String thirdSubscriptionId) {
        log.info("订阅取消续订，appstore订阅id：{}", thirdSubscriptionId);
        // 根据第三放订阅id查询对应的用户id和产品id
        webhookService.handleUnsubscribeFollowUpActions(thirdSubscriptionId);
    }
}
