package cn.kdan.cloud.pdf.office.payment.service.impl;

import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.payment.bo.*;
import cn.kdan.cloud.pdf.office.api.payment.constant.OrderConstant;
import cn.kdan.cloud.pdf.office.api.payment.constant.PaddleAPIConstant;
import cn.kdan.cloud.pdf.office.api.payment.constant.UserAction;
import cn.kdan.cloud.pdf.office.api.payment.dto.ChargebackOrderDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreateOrderDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreateOrderManualDTO;
import cn.kdan.cloud.pdf.office.api.payment.enums.PaymentMethodEnum;
import cn.kdan.cloud.pdf.office.api.payment.enums.ProductTypeEnum;
import cn.kdan.cloud.pdf.office.api.payment.paypal.ApplicationContext;
import cn.kdan.cloud.pdf.office.api.payment.paypal.transaction.PaypalTransaction;
import cn.kdan.cloud.pdf.office.api.payment.paypal.transaction.TransactionsItem;
import cn.kdan.cloud.pdf.office.api.payment.vo.CreateOrderResultVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrderUpdateVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.constant.RabbitMqConstant;
import cn.kdan.cloud.pdf.office.common.constant.RedisKeyConstant;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.payment.client.AppStoreClient;
import cn.kdan.cloud.pdf.office.payment.client.PaddleClient;
import cn.kdan.cloud.pdf.office.payment.client.PaypalClient;
import cn.kdan.cloud.pdf.office.payment.entity.Order;
import cn.kdan.cloud.pdf.office.payment.enums.PaypalTransactionStatusEnum;
import cn.kdan.cloud.pdf.office.payment.enums.PaypalWebhookEventEnum;
import cn.kdan.cloud.pdf.office.payment.error.ErrorMessage;
import cn.kdan.cloud.pdf.office.payment.mapper.OrderMapper;
import cn.kdan.cloud.pdf.office.payment.properties.PaypalProperties;
import cn.kdan.cloud.pdf.office.payment.service.OrderService;
import cn.kdan.cloud.pdf.office.payment.service.WebhookService;
import com.alibaba.nacos.common.utils.UuidUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.BeanUtils;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.redis.cache.RedisCache;
import org.springframework.data.redis.cache.RedisCacheManager;
import org.springframework.security.access.method.P;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author ComPDFKit-WPH 2023-01-29
 * <p>
 * OrdersServiceImpl
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class OrderServiceImpl extends ServiceImpl<OrderMapper, Order> implements OrderService {

    private final ProductApi productApi;

    private final PaddleClient paddleClient;

    private final RabbitTemplate rabbitTemplate;

    private final AppStoreClient appStoreClient;

    private final PaypalClient paypalClient;

    private final PaypalProperties paypalProperties;

    private final UserApi userApi;


    @Resource(name = "myRedisCacheManager")
    private RedisCacheManager myRedisCacheManager;

    @Override
    @Transactional
    @Cacheable(value = RedisKeyConstant.CACHE_PAYMENT_CREAT_ORDER + "#3#h",
            key = "T(String).valueOf(#createOrderDTO.email)" +
                    ".concat('-').concat(#createOrderDTO.productId)" +
                    ".concat('-').concat(#createOrderDTO.paymentMethod.getValue())",
            cacheManager = "myRedisCacheManager", unless = "#result == null")
    public CreateOrderResultVO createOrder(CreateOrderDTO createOrderDTO) {
        //判断是否有首年优惠
        ProductVO product = productApi.getProductForPay(createOrderDTO.getProductId(),createOrderDTO.getPaymentMethod().getValue(),createOrderDTO.getIsDiscount()).getResult();
        PaymentMethodEnum paymentMethodEnum = createOrderDTO.getPaymentMethod();
        log.info("createOrder:{}", createOrderDTO);
        Order order = new Order();
        order.setAppId(product.getAppId().toString());
        if(StringUtils.isNotEmpty(createOrderDTO.getUserId())){
            order.setUserId(createOrderDTO.getUserId());
        }
        order.setEmail(createOrderDTO.getEmail());
        order.setProductId(createOrderDTO.getProductId());
        order.setPayment(paymentMethodEnum.getValue());
        order.setStatus(OrderConstant.CREATED);
        order.setInvoiceNo(MyDateUtils.getTimeStamp() + (int) ((Math.random() * 9 + 1) * 1000));
        String readeNo = MyDateUtils.getTimeStamp() + "-" + createOrderDTO.getProductId() + "-" + (int) ((Math.random() * 9 + 1) * 1000);
        order.setTradeNo(readeNo);
        switch (paymentMethodEnum) {
            case WXPAY:
                break;
            case ALIPAY:
                break;
            case PADDLE:
                //设置折扣价格
                BigDecimal price = createOrderDTO.getIsDiscount() == 0 ? product.getPrice():product.getDisplayPrice();
                GeneratePayLinkPaddleBO paddleBO = new GeneratePayLinkPaddleBO();
                paddleBO.setPrices(new String[]{"USD:" + price.toString()});
                paddleBO.setProductId(product.getThirdPayId());
                paddleBO.setPassthrough(readeNo);
                if(StringUtils.isNotEmpty(createOrderDTO.getCountryCode())){
                    paddleBO.setCustomerCountry(createOrderDTO.getCountryCode());
                }
                if(StringUtils.isNotEmpty(createOrderDTO.getEmail())){
                    paddleBO.setCustomerEmail(createOrderDTO.getEmail());
                }
                if(StringUtils.isNotEmpty(createOrderDTO.getPostCode())){
                    paddleBO.setCustomerPostcode(createOrderDTO.getPostCode());
                }
                GeneratePayLinkResultPaddleBO resultPaddleBO = paddleClient.createSubscriptions(paddleBO);
                order.setPagePayUrl(resultPaddleBO.getUrl());
                order.setResult(paddleBO.toString());
                order.setThirdOrderNo(paddleBO.getPassthrough());
                //订单类型1网站购买
                order.setType(1);
                //设置第三方订单
                order.setThirdTradeNo(paddleBO.getPassthrough());
                //设置支付方式
                order.setPayment(paymentMethodEnum.getValue());
                break;
            case NO_NEED_TO_PAY:
                break;
            case PAYPAL:
                //创建paypal订阅
                if(ProductTypeEnum.PLAN.getValue().equals(product.getType())) {
                    CreateSubscriptionsPaypalBO createSubscriptionsBO = new CreateSubscriptionsPaypalBO();
                    ApplicationContext applicationContext = new ApplicationContext();
                    applicationContext.setBrand_name("KDanMobile");
                    applicationContext.setLocale("zh-CN");
                    applicationContext.setUser_action(UserAction.SUBSCRIBE_NOW);
                    //成功或失败回调url
                    applicationContext.setReturn_url(paypalProperties.getReturnUrl());
                    applicationContext.setCancel_url(paypalProperties.getCancelUrl());
                    //订阅计划id
                    createSubscriptionsBO.setPlan_id(product.getThirdPayId());
                    createSubscriptionsBO.setApplication_context(applicationContext);
                    CreateSubscriptionsResultPaypalBO resultBO = paypalClient.createSubscriptions(createSubscriptionsBO);
                    order.setResult(resultBO.toString());
                    order.setThirdOrderNo(resultBO.getId());
                    //订单类型1网站购买
                    order.setType(1);
                    //设置第三方订单
                    order.setThirdTradeNo(resultBO.getId());
                    //设置支付方式
                    order.setPayment(paymentMethodEnum.getValue());
                    order.setPagePayUrl(resultBO.getLinks().get(0).getHref());
                };
                break;
            // 当前支付方式无效
            default:
                throw new BackendRuntimeException(ErrorMessage.PAYMENT_METHOD_INVALID);
        }
        this.baseMapper.insert(order);
        CreateOrderResultVO createOrderResultVO = new CreateOrderResultVO();
        BeanUtils.copyProperties(order, createOrderResultVO);
        // 订单关闭
        log.info("rabbitMq 发送延时队列，关闭订单：{}", order.getId());
        rabbitTemplate.convertAndSend(RabbitMqConstant.PAYMENT_DELAY_EXCHANGE,
                RabbitMqConstant.PAYMENT_ORDER_ROUTING_KEY,
                order.getId(),
                (msg -> {
                    msg.getMessageProperties().setHeader("x-delay", ( 3 * 60 * 60 * 1000) + "");
                    return msg;
                }));
        return createOrderResultVO;
    }

    @Override
    @Transactional
    public CreateOrderResultVO appStoreCreateOrder(CreateOrderDTO createOrderDTO) {
        PaymentMethodEnum paymentMethodEnum = createOrderDTO.getPaymentMethod();
        log.info("createOrder:{}", createOrderDTO);
        Order order = new Order();
        order.setEmail(userApi.getById(createOrderDTO.getUserId()).getResult().getEmail());
//        order.setAppId(creatOrderDTO.getAppId());
//        order.setUserId(creatOrderDTO.getUserId());
//        order.setProductId(creatOrderDTO.getProductId());
        order.setAppId(createOrderDTO.getAppId());
        order.setUserId(createOrderDTO.getUserId());
        order.setProductId(createOrderDTO.getProductId());
        order.setPayment(paymentMethodEnum.getValue());
        order.setStatus(OrderConstant.CREATED);
        order.setInvoiceNo(MyDateUtils.getTimeStamp() + (int) ((Math.random() * 9 + 1) * 1000));
//        orders.setDiscount(createOrderDTO.getDiscount());
        String readeNo = MyDateUtils.getTimeStamp() + "-" + createOrderDTO.getProductId() + "-" + (int) ((Math.random() * 9 + 1) * 1000);
        order.setTradeNo(readeNo);
        //订单类型3appstore购买
        order.setType(3);
        this.baseMapper.insert(order);
        CreateOrderResultVO createOrderResultVO = new CreateOrderResultVO();
        createOrderResultVO.setUuid(UuidUtils.generateUuid());
        BeanUtils.copyProperties(order, createOrderResultVO);
//        // 订单关闭
//        log.info("rabbitMq 发送延时队列，关闭订单：{}", order.getId());
//        rabbitTemplate.convertAndSend(RabbitMqConstant.PAYMENT_DELAY_EXCHANGE,
//                RabbitMqConstant.PAYMENT_ORDER_ROUTING_KEY,
//                order.getId(),
//                (msg -> {
//                    msg.getMessageProperties().setHeader("x-delay", ( 3 * 60 * 60 * 1000) + "");
//                    return msg;
//                }));
        return createOrderResultVO;
    }

    @Override
    public void closeOrder(String orderId) {
        OrdersVO orders = getOrderById(orderId);
        log.info("当前订单信息：{}", orders);
        if (ObjectUtils.isNotEmpty(orders.getStatus())&&orders.getStatus().equals(OrderConstant.CREATED)) {
            Order order = new Order();
            order.setStatus(OrderConstant.CLOSED);
            order.setId(orderId);
            this.baseMapper.updateById(order);
        }
        //删除缓存中的订单
        deleteTheCacheInTheCacheManager(orders.getEmail() + "-" + orders.getProductId() + "-" + orders.getPayment());
    }

    @Override
    public OrderUpdateVO updateOrderSucceeded(OrderSucceededBO orderSucceeded) {
        Order orders = new Order();
        orders.setThirdOrderNo(orderSucceeded.getThirdOrderNo());
        orders.setThirdTradeNo(orderSucceeded.getThirdTradeNo());
        orders.setPrice(orderSucceeded.getPrice());
        orders.setUserId(orderSucceeded.getUserId());
        orders.setPayDate(new Date());
        orders.setSubscriptionId(orderSucceeded.getSubscriptionId());
        orders.setStatus(OrderConstant.COMPLETED);
        Order orderTradeNo;
        if (ObjectUtils.isNotEmpty(orderSucceeded.getOrderId())) {
            this.baseMapper.update(orders, new LambdaQueryWrapper<Order>()
                    .eq(Order::getId, orderSucceeded.getOrderId()));
            orderTradeNo = this.baseMapper.selectOne(new LambdaQueryWrapper<Order>()
                    .select(Order::getId, Order::getUserId,Order::getProductId,Order::getAppId,Order::getPayment,Order::getEmail)
                    .eq(Order::getId, orderSucceeded.getOrderId()));
        }else {
            this.baseMapper.update(orders, new LambdaQueryWrapper<Order>()
                    .eq(Order::getThirdTradeNo, orderSucceeded.getThirdTradeNo()));
            orderTradeNo = this.baseMapper.selectOne(new LambdaQueryWrapper<Order>()
                    .select(Order::getId, Order::getUserId,Order::getProductId,Order::getAppId,Order::getPayment,Order::getEmail)
                    .eq(Order::getThirdTradeNo, orderSucceeded.getThirdTradeNo()));
        }
        //删除缓存中的订单
        deleteTheCacheInTheCacheManager(orderTradeNo.getEmail() + "-" + orderTradeNo.getProductId() + "-" + orderTradeNo.getPayment());
        return OrderUpdateVO.builder()
                .orderId(orderTradeNo.getId())
                .userId(orderTradeNo.getUserId())
                .productId(orderTradeNo.getProductId())
                .appId(orderTradeNo.getAppId())
                .build();
    }

    /**
     * 删除redis缓存管理器中的对应key的缓存
     *
     * @param key key
     */
    private void deleteTheCacheInTheCacheManager(String key) {
        log.info("删除缓存中的订单 key"+ key);
        RedisCache cache = (RedisCache) myRedisCacheManager.getCache(RedisKeyConstant.CACHE_PAYMENT_CREAT_ORDER);
        if (cache != null) {
            cache.evictIfPresent(key);
        }
    }

    @Override
    public OrdersVO getOrderById(String orderId) {
        OrdersVO ordersVO = new OrdersVO();
        Order orders = this.baseMapper.selectOne(new LambdaQueryWrapper<Order>().eq(Order::getId, orderId));
        if(ObjectUtils.isEmpty(orders)){
            return null;
        }
        BeanUtils.copyProperties(this.baseMapper.selectOne(new LambdaQueryWrapper<Order>().eq(Order::getId, orderId)), ordersVO);
        return ordersVO;
    }


    @Override
    public List<OrdersVO> getOrderByTradeNo(String tradeNo) {
        return this.baseMapper.selectOrdersByTradeNo(tradeNo);
    }


    @Override
    public OrderUpdateVO createOrderManual(CreateOrderManualDTO createOrderManual) {
        Order orders = new Order();
        orders.setId(createOrderManual.getId());
        orders.setAppId(createOrderManual.getAppId());
        orders.setProductId(createOrderManual.getProductId());
        orders.setUserId(createOrderManual.getUserId());
        orders.setTradeNo(createOrderManual.getTradeNo());
        orders.setThirdTradeNo(createOrderManual.getThirdTradeNo());
        if(!StringUtils.isEmpty(createOrderManual.getThirdOrderNo())){
            orders.setThirdOrderNo(createOrderManual.getThirdOrderNo());
        }
        orders.setEmail(createOrderManual.getEmail());
        orders.setSubscriptionId(createOrderManual.getSubscriptionId());
        orders.setPrice(createOrderManual.getPrice());
        orders.setPayment(createOrderManual.getPaymentMethod().getValue());
        orders.setStatus(OrderConstant.COMPLETED);
        orders.setPayDate(new Date());
        this.baseMapper.insert(orders);
        return OrderUpdateVO.builder()
                .userId(createOrderManual.getUserId())
                .orderId(orders.getId())
                .build();
    }


    @Override
    public void chargebackOrder(@NotNull ChargebackOrderDTO chargebackOrderDTO) {
        log.info("手动退单修改状态：{}", chargebackOrderDTO);
        // 修改订单状态
        Order orders = new Order();
        orders.setId(chargebackOrderDTO.getOrderId());
        orders.setStatus(OrderConstant.REFUND);
        this.baseMapper.updateById(orders);
        log.info("手动退单修改状态成功：{}", chargebackOrderDTO);
    }

    @Override
    public void insertOrder(Order order) {
        this.baseMapper.insert(order);
    }

    @Override
    public List<OrdersVO> getOrderByUserId(String userId,String appId) {
        return this.baseMapper.selectList(new LambdaQueryWrapper<Order>()
                .eq(Order::getUserId, userId).eq(Order::getAppId, appId)
                .orderByDesc(Order::getCreatedAt))
                .stream().map(order -> {
                    OrdersVO ordersVO = new OrdersVO();
                    BeanUtils.copyProperties(order, ordersVO);
                    return ordersVO;
                }).collect(Collectors.toList());
    }



    @Override
    public OrdersVO getOrderByThirdOrderNo(String thirdOrderNo) {
        OrdersVO ordersVO = new OrdersVO();
        Order order = this.baseMapper.selectOne(new LambdaQueryWrapper<Order>().eq(Order::getThirdOrderNo,thirdOrderNo));
        if(ObjectUtils.isNotEmpty(order)){
            BeanUtils.copyProperties(this.baseMapper.selectOne(new LambdaQueryWrapper<Order>().eq(Order::getThirdOrderNo,thirdOrderNo)),ordersVO);
            return ordersVO;
        }else {
            return null;
        }
    }



}
