package cn.kdan.cloud.pdf.office.product.controller;

import cn.kdan.cloud.pdf.office.api.product.dto.ProductListDTO;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductPriceVo;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.product.entity.Product;
import cn.kdan.cloud.pdf.office.product.service.ProductService;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * @author ComPDFKit-WPH 2023/2/1
 */
@RestController
@RequestMapping("/product")
@RequiredArgsConstructor
public class ProductController {

    private final ProductService productService;

    @GetMapping("/getProductPrice")
    public ResultMap<ProductPriceVo> getProductPrice(@Validated @NotNull(message = "产品id不能为空") @RequestParam("productId") String productId,
                                                     @NotNull(message = "userId不能为空") @RequestParam("userId") String userId) {

        return ResultMap.success(productService.getProductPrice(productId, userId));
    }


    @GetMapping("/getProduct")
    public ResultMap<ProductVO> getProduct(@Validated @NotNull(message = "产品id不能为空") @RequestParam("productId") String productId) {
        return ResultMap.success(productService.getProduct(productId));
    }
    /**
     * 获取产品（带第三方支付id）
     * @param productId
     * @param payment
     * @param isDiscount
     * @return
     */
    @GetMapping("/getProductForPay")
    public ResultMap<ProductVO> getProductForPay(@Validated @NotNull(message = "产品id不能为空") @RequestParam("productId") String productId,
                                                 @Validated @NotNull(message = "支付方式不能为空") @RequestParam("payment") Integer payment,
                                                 @Validated @NotNull(message = "是否首年优惠不能为空") @RequestParam("isDiscount") Integer isDiscount) {
        return ResultMap.success(productService.getProductForPay(productId,payment,isDiscount ));
    }


    /**
     * 获取产品列表
     *
     * @param size  size
     * @param page  page
     * @param appId appId
     * @return ProductVO
     */
    @GetMapping("/getProductList")
    public ResultMap<IPage<ProductVO>> getProductList(@RequestParam Long size, @RequestParam Long page,
                                                      @RequestParam String appId) {
        ProductListDTO productListDTO = new ProductListDTO();
        productListDTO.setSize(size);
        productListDTO.setPage(page);
        productListDTO.setAppId(appId);
        return ResultMap.success(productService.page(productListDTO));
    }

    /**
     * 获取新用户专用活动
     *
     * @param appId appId
     * @return 新用户专用活动
     */
    @GetMapping("/getNewUserProduct")
    public ResultMap<List<ProductVO>> getNewUserProduct(@RequestParam String appId) {
        return ResultMap.success(productService.getNewUserProduct(appId));
    }

    @GetMapping("/getProductByAppIdAndModel")
    public ResultMap<ProductVO> getProductByAppIdAndModel(@RequestParam String appId,@RequestParam Integer platform) {
        return ResultMap.success(productService.getProductByAppIdAndModel(appId,platform));
    }

    @GetMapping("/getProductByThirdPayId")
    public ResultMap<ProductVO> getProductByThirdPayId(@RequestParam("thirdPayId") String thirdPayId){
        return ResultMap.success(productService.getProductByThirdPayId(thirdPayId));
    }

}
