package cn.kdan.cloud.pdf.office.product.service.impl;

import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.utils.SubscriptionUtil;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.vo.UserSubscriptionInfoVO;
import cn.kdan.cloud.pdf.office.api.product.dto.ProductListDTO;
import cn.kdan.cloud.pdf.office.api.product.vo.ActivityInfoVO;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductPriceVo;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.JsonUtils;
import cn.kdan.cloud.pdf.office.product.entity.Product;
import cn.kdan.cloud.pdf.office.product.entity.RProductPay;
import cn.kdan.cloud.pdf.office.product.error.ErrorMessage;
import cn.kdan.cloud.pdf.office.product.mapper.ProductMapper;
import cn.kdan.cloud.pdf.office.product.service.ActivityInformationService;
import cn.kdan.cloud.pdf.office.product.service.ProductService;
import cn.kdan.cloud.pdf.office.product.service.RProductPayService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author ComPDFKit-WPH 2023-01-28
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class ProductServiceImpl extends ServiceImpl<ProductMapper, Product> implements ProductService {

    private final ActivityInformationService activityService;

    private final UserApi userAPi;


    private final RProductPayService rProductPayService;

    @Override
    public IPage<ProductVO> page(ProductListDTO query) {
        Page<Product> productPage = this.page(new Page<>(query.getPage(), query.getSize()), new LambdaQueryWrapper<Product>()
                .eq(Product::getAppId, query.getAppId()));
        List<ProductVO> productVOS = JsonUtils.jsonStringToList(JsonUtils.getJsonString(productPage.getRecords()), ProductVO.class);
        productVOS.forEach(productVO -> {
            if (!ObjectUtils.isEmpty(productVO.getActivityId())) {
                productVO.setActivityInfo(activityService.getActivityInfoByActivityId(productVO.getActivityId()));
            }
        });
        Page<ProductVO> page = new Page<>(query.getPage(), query.getSize());
        page.setRecords(productVOS);
        return page;
    }

    @Override
    public ProductPriceVo getProductPrice(String productId, String userId) {
        Product product = this.baseMapper.selectOne(new LambdaQueryWrapper<Product>()
                .select(Product::getPrice, Product::getCnyPrice, Product::getPaddleProductId, Product::getActivityId)
                .eq(Product::getId, productId));
        ProductPriceVo productPriceVo = new ProductPriceVo();
        productPriceVo.setPaddleProductId(product.getPaddleProductId());
        // 判断当前产品是否有活动 获取活动的最终价格
        if (!ObjectUtils.isEmpty(product.getActivityId())) {
            ActivityInfoVO activityInfo = activityService.getActivityInfoByActivityId(product.getActivityId());
            // 判断是否是新用户专用
            List<UserSubscriptionInfoVO> subscriptionInfoList = userAPi.getInfoById(userId).getResult().getSubscriptionInfoList();
            if (activityInfo.getActivityType() == 1) {
                UserSubscriptionInfoVO userSubscriptionInfoVO = SubscriptionUtil.getUserSubscriptionInfoByPlatform(subscriptionInfoList, product.getPlatform());
                if (ObjectUtils.isEmpty(userSubscriptionInfoVO)) {
                    productPriceVo.setPrice(activityInfo.getFinalPrice());
                    productPriceVo.setCnyPrice(activityInfo.getFinalCnyPrice());
                } else {
                    throw new BackendRuntimeException(ErrorMessage.USER_IS_NOT_NEW_USER);
                }
            } else {
                productPriceVo.setPrice(activityInfo.getFinalPrice());
                productPriceVo.setCnyPrice(activityInfo.getFinalCnyPrice());
            }
        }
        return productPriceVo;
    }

    @Override
    public ProductVO getProduct(String productId) {
        ProductVO productVO = new ProductVO();
        BeanUtils.copyProperties(this.baseMapper.selectOne(new LambdaQueryWrapper<Product>().eq(Product::getId, productId)), productVO);
        return productVO;
    }

    @Override
    public ProductVO getProductByAppIdAndModel(String appId, Integer model) {
        List<Product> list = this.baseMapper.selectList(new LambdaQueryWrapper<Product>().eq(Product::getAppId, appId).eq(Product::getPlatform, model));
        ProductVO productVO = new ProductVO();
        BeanUtils.copyProperties(list.get(0),productVO);
        return productVO;
    }

    @Override
    public List<ProductVO> getNewUserProduct(String appId) {
        List<ProductVO> productVOS = this.baseMapper.selectNewUserProductByAppId(appId);
        for (ProductVO productVO : productVOS) {
            productVO.setActivityInfo(activityService.getActivityInfoByActivityId(productVO.getActivityId()));
        }
        return productVOS;
    }

    @Override
    public ProductVO getProductForPay(String productId, Integer payment, Integer isDiscount) {
        ProductVO productVO = new ProductVO();
        BeanUtils.copyProperties(this.baseMapper.selectOne(new LambdaQueryWrapper<Product>().eq(Product::getId, productId)), productVO);
        RProductPay productPay = new RProductPay();
        productPay.setProductId(productId);
        productPay.setPayment(payment);
        productPay.setIsDiscount(isDiscount);
        List<RProductPay>  list = rProductPayService.list(productPay);
        productVO.setThirdPayId(list.get(0).getThirdPayId());
        return productVO;
    }

    @Override
    public ProductVO getProductByThirdPayId(String thirdPayId) {
        ProductVO productVO = this.baseMapper.getProductByThirdPayId(thirdPayId);
        if (ObjectUtils.isEmpty(productVO)) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_THIRD_PAY_ID_ILLEGAL);
        }
        return productVO;
    }

}
