package cn.kdan.cloud.pdf.office.sso.controller;

import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;

import cn.kdan.cloud.pdf.office.api.payment.feign.SubscriptionApi;
import cn.kdan.cloud.pdf.office.common.enums.account.PayTypeEnum;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.common.vo.UserSubscriptionInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import cn.kdan.cloud.pdf.office.sso.pojo.LogOutSystemDTO;
import cn.kdan.cloud.pdf.office.sso.service.AuthService;
import cn.kdan.cloud.pdf.office.common.pojo.CustomUserDetails;
import cn.kdan.cloud.pdf.office.sso.utils.NullToEmptyStringConverter;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.authentication.OAuth2AuthenticationDetails;
import org.springframework.web.bind.annotation.*;
import rx.Subscription;

import javax.servlet.http.HttpServletRequest;
import java.security.Principal;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/user")
public class UserController {

    @Autowired
    private UserApi userApi;
    @Autowired
    private ProductApi productApi;
    @Autowired
    private SubscriptionApi subscriptionApi;
    @Autowired
    private AuthService authService;

    @GetMapping("/me")
    public UserInfoVO me(Principal principal) {
        if (principal instanceof OAuth2Authentication) {
            {
                String userId = ((CustomUserDetails)((OAuth2Authentication)principal).getPrincipal()).getId();
                authService.refreshToken(principal);
                // 获取用户的基本信息
                UserInfoVO user = userApi.getInfoById(userId).getResult();
                user.getSubscriptionInfoList().forEach(item->{
                    ProductVO vo = productApi.getProduct(item.getProductId()).getResult();
                    item.setProductCode(vo.getCode());
                });
                List<UserSubscriptionInfoVO> list = user.getSubscriptionInfoList().stream()
                        .sorted(Comparator.comparing(UserSubscriptionInfoVO::getPlatform, Comparator.reverseOrder()))
                        .collect(Collectors.toList());
                list.forEach(item->{
                    //自动续订，显示续订日期
                    if(!ObjectUtils.isEmpty(item.getEndDate())&&PayTypeEnum.AUTO.value().equals(item.getPayType())){
                        LocalDateTime payTime = subscriptionApi.getNextPayTime(item.getUserId(),item.getProductId()).getResult();
                        Date date = MyDateUtils.localDateTimeToDate(payTime);
                        item.setEndDate(date);
                    }
                    // 没有购买显示当前时间
                    if(ObjectUtils.isEmpty(item.getEndDate())){
                        item.setEndDate(new Date());
                    }
                });
                user.setSubscriptionInfoList(list);
                user.setDigestPassword(null);
                return user;
            }
        }
        return null;
    }

    /**
     * 登出(登出设备)
     *
     * @param deviceSign 设备号
     * @param appId appId
     * @param request 请求
     * @param principal 凭证
     * @return Boolean
     */
    @PostMapping("/logout")
    public ResultMap<Boolean> logout(@RequestParam String deviceSign, @RequestParam String appId, HttpServletRequest request, Principal principal) {
        String token = getToken(principal);
        String userId = getUserId(principal);
        authService.logout(token,userId,deviceSign,appId,true);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.TRUE);
    }

    /**
     * 用户自己手动注销
     *
     * @param code
     * @return
     */
    @PostMapping("/logOffForUser")
    public ResultMap<Boolean> logOffForUser(@RequestParam String code,@RequestParam String appId,  Principal principal) {
        String userId = getUserId(principal);
        userApi.logOffForUser(code,userId, appId).getResult();
        UserVO userVO = userApi.getById(userId).getResult();
        authService.logOutAll(userVO);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS,Boolean.TRUE);
    }

    @PostMapping("/logOutAllSystem")
    public ResultMap<Boolean> logOutAllSystem(@RequestBody LogOutSystemDTO logOutSystemDTO) {
        UserVO userVO = userApi.getById(logOutSystemDTO.getUserId()).getResult();
        authService.logOutAll(userVO);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS,Boolean.TRUE);
    }

    @PostMapping("/logOutSystem")
    public ResultMap<Boolean> logOutSystem(@RequestBody LogOutSystemDTO logOutSystemDTO) {
        authService.logOutDevice(logOutSystemDTO.getAppId(),logOutSystemDTO.getUserId(),logOutSystemDTO.getUniqueSn());
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS,Boolean.TRUE);
    }


    private String getUserId(Principal principal) {
        if (principal instanceof OAuth2Authentication) {
            return ((CustomUserDetails) (((OAuth2Authentication) principal).getPrincipal())).getId();
        }
        throw new BackendRuntimeException(CommonConstant.EXCEPTION_MSG_LOGIN_USER_NOT_FOUND);
    }

    private String getToken(Principal principal) {
        if (principal instanceof OAuth2Authentication) {
            return ((OAuth2AuthenticationDetails) (((OAuth2Authentication) principal).getDetails())).getTokenValue();
        }
        throw new BackendRuntimeException(CommonConstant.EXCEPTION_MSG_LOGIN_USER_NOT_FOUND);
    }
}
