package cn.kdan.cloud.pdf.office.sso.service;


import cn.kdan.cloud.pdf.office.common.dto.UserRegisterDTO;
import cn.kdan.cloud.pdf.office.common.dto.UserResetPasswordDTO;
import cn.kdan.cloud.pdf.office.common.dto.ValidUserDTO;
import cn.kdan.cloud.pdf.office.common.enums.EmailCodeTypeEnum;
import cn.kdan.cloud.pdf.office.common.vo.TokenVO;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import cn.kdan.cloud.pdf.office.sso.enums.VerifyTypeEnum;

import javax.servlet.http.HttpServletRequest;
import java.security.Principal;

public interface AuthService {


    String getAuthorizeCode(String userId, String clientId, HttpServletRequest request);

    TokenVO getToken(String code, String clientId, HttpServletRequest request);

    /**
     * 邮箱登录
     *
     * @param email 邮箱
     * @param password 验证码
     * @param appId appid
     * @param platformType  1官网 2后台
     * @param request
     * @return TokenVO
     */
    TokenVO emailLogin(String email, String password, String appId, String platformType, String deviceSign, String model,HttpServletRequest request);

    /**
     * 登出所有设备
     *
     * @param userVO
     */
    void logOutAll (UserVO userVO);

    /**
     * param
     *
     * @param param param
     */
    void validUserRegisterParam(ValidUserDTO param);

    void validUserLoginParam(ValidUserDTO param);

    /**
     * 发送验证码
     *
     * @param action  EmailCodeTypeEnum
     * @param type VerifyTypeEnum
     * @param receiver 邮箱
     * @param appId appId 应用id
     * @return boolean
     */
    boolean getVerifyCode(EmailCodeTypeEnum action, VerifyTypeEnum type, String receiver, String appId);

    /**
     * 登出（token失效，清除登录设备信息）
     *
     * @param token      token
     * @param userId     用户id
     * @param deviceSign 设备号
     * @param appId      appId
     * @param isDelete
     */
    void logout(String token, String userId, String deviceSign, String appId, Boolean isDelete);

    /**
     * 登出固定设备
     *
     */
    void logOutDevice(String appId, String userId, String uniqueSn);

    /**
     * 检查账号是否在其他设备登录，设备是否被自动登出
     * @param deviceSign  设备编号
     * @param appId 应用id
     */
    void checkDeviceStatus(String deviceSign, String appId);

    /**
     * 登录时判断邮箱验证码是否正确
     * @param type 类型
     * @param account 账户
     * @param code 验证码
     */
    void isEmailCodeValid(EmailCodeTypeEnum type, String account, String code,  String appId);

    void refreshToken(Principal principal);

    /**
     * pdf office用户注册 注册后自动登录
     * @param userRegisterDTO
     * @return
     */
    TokenVO emailRegister(UserRegisterDTO userRegisterDTO);

    /**
     * 重置密码
     *
     * @param userResetPasswordDTO userResetPasswordDTO
     */
    void resetPassword(UserResetPasswordDTO userResetPasswordDTO);
}
